/**
 * Copyright (c) 2005-2012 Fossil E-Commerce, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package javax.jbi.management;

/**
 * The deployment service MBean allows administrative tools to manage service
 * assembly deployments. The tasks supported are:
 * <ul>
 *   <li>Deploying a service assembly.
 *   <li>Undeploying a previously deployed service assembly.
 *   <li>Querying deployed service assemblies:
 *   <ul>
 *     <li>For all components in the system.
 *     <li>For a particular component.
 *   </ul
 *   <li>Control the state of deployed service assemblies:
 *   <ul>
 *     <li>Start the service units that contained in the SA.
 *     <li>Stop the service units that contained in the SA.
 *     <li>Shut down the service units that contained in the SA.
 *   </ul>
 *   <li>Query the service units deployed to a particular component.
 *   <li>Check if a service unit is deployed to a particular component.
 *   <li>Query the deployment descriptor for a particular service assembly.
 * </ul>
 *
 * @author JSR208 Expert Group
 */
public interface DeploymentServiceMBean {

	/**
     * The service assembly has been deployed, or shutdown
     */	
	public static final java.lang.String SHUTDOWN = "Shutdown";

    /**
     * The service assembly is started. This means that the assembly's offered
     * services can accept message exchanges, and it can send exchanges to
     * consume services.
     */
    public static final java.lang.String STARTED = "Started";

	/**
	 * The service assembly is stopped. This means that the assembly's offered
	 * services can accept message exchanges, but it will not send any.
	 */
	public static final java.lang.String STOPPED = "Stopped";


	/**
     * Returns <code>true</code> if the the given component accepts the
     * deployment of service units. This is used by admin tools to determine
     * which components can be named in service assembly deployment descriptors.
     * 
     * @param componentName
     *            name of the component; must be non-null and non-empty
     * @return <code>true</code> if the named component accepts deployments;
     *         <code>false</code> if the named component does not accept
     *         deployments or it does not exist
     */
	public boolean canDeployToComponent(java.lang.String componentName);
	
	/**
	 * TODO deploy verify that all Service units are present in the package and that all targeted components are installed in the JBI System 
	 * TODO check duplicate service units as describe in spec section 6.6
	 * TODO deploy add the result of deploy method call on su to the management message "result"
	 * TODO deploy verify deployment success before registration
	 * TODO deploy returning xml result instead of SA name
	 * 
     * Deploys the given Service Assembly to the JBI environment.
     * <p>
     * Note that the implementation cannot automatically start the service
     * assembly after deployment; it must wait for the
     * {@link DeploymentServiceMBean#start(String)} method to be invoked by the
     * administrative tool.
     * 
     * @param serviceAssemblyZipUrl
     *            String containing the location URL of the Service Assembly ZIP
     *            file; must be non-null, non-empty, and a legal URL.
     * @return Result/Status of the current deployment; must conform to JBI
     *         management result/status XML schema; must be non-null and
     *         non-empty.
     * @throws java.lang.Exception
     *             if complete deployment fails
     */
    public java.lang.String deploy(java.lang.String serviceAssemblyZipUrl)
            throws java.lang.Exception;

	/**
	 * Returns an array of component names, where for each the given assembly
	 * contains a service unit for the component.
	 * 
	 * @param serviceAssemblyName
	 *            the service assembly to be queried; must be non-null and
	 *            non-empty
	 * @return array of component names, where for each name the given assembly
	 *         contains a service unit from the given service assembly; must be
	 *         non-null; may be empty
	 * @throws java.lang.Exception
	 *             if a processing error occurs
	 */
	public java.lang.String[] getComponentsForDeployedServiceAssembly(
			java.lang.String serviceAssemblyName) throws java.lang.Exception;
	
	/**
     * Returns a list of Service Assemblies deployed to the JBI environment.
     * 
     * @return list of Service Assembly names; must be non-null; may be empty
     * @throws java.lang.Exception
     *             if a processing error occurs
     */
	public java.lang.String[] getDeployedServiceAssemblies()
            throws java.lang.Exception;
	
	/**
     * Returns an array of Service Assembly names, where each assembly contains
     * Service Units for the given component.
     * 
     * @param componentName
     *            name of the component to query; must be non-null and non-empty
     * @return array of of Service Assembly names, where each assembly contains
     *         a Service Unit for the named component; must be non-null; may be
     *         empty
     * @throws java.lang.Exception
     *             if a processing error occurs
     */
    public java.lang.String[] getDeployedServiceAssembliesForComponent(
            java.lang.String componentName) throws java.lang.Exception;
	
	/**
     * Returns an array of service unit names that are currently deployed to the
     * named component.
     * 
     * @param componentName
     *            the name of the component to query; must be non-null and
     *            non-empty
     * @return array of service unit names deployed in the named component; must
     *         be non-null; may be empty
     * @throws java.lang.Exception
     *             if a processing error occurs
     */

    public java.lang.String[] getDeployedServiceUnitList(
            java.lang.String componentName) throws java.lang.Exception;
	
	/**
     * Returns the deployment descriptor of the Service Assembly that was
     * deployed to the JBI enviroment, serialized to a <code>String</code>.
     *
     * @param serviceAssemblyName
     *            name of the service assembly to be queried; must be non-null
     *            and non-empty
     * @return descriptor of the Assembly Unit; must be non-null
     * @throws java.lang.Exception
     *             if a processing error occurs getState(String)
     */
    public java.lang.String getServiceAssemblyDescriptor(
            java.lang.String serviceAssemblyName) throws java.lang.Exception;
	
	/**
     * Get the running state of a service assembly. The possible result values
     * of this query are enumerated by the following set of constants:
     * {@link DeploymentServiceMBean#SHUTDOWN},
     * {@link DeploymentServiceMBean#STOPPED},
     * {@link DeploymentServiceMBean#STARTED}.
     * 
     * @param serviceAssemblyName
     *            name of the assembly to query; must be non-null and non-empty
     * @return the state of the service assembly, as a string value; must be one
     *         of the enumerated string values provided by this interface:
     *         {@link DeploymentServiceMBean#SHUTDOWN},
     *         {@link DeploymentServiceMBean#STOPPED}, or
     *         {@link DeploymentServiceMBean#STARTED}.
     * @return java.lang.Exception - if there is no such assembly
     */
    public java.lang.String getState(java.lang.String serviceAssemblyName)
            throws java.lang.Exception;
	
	/**
     * Queries if the named Service Unit is currently deployed to the named
     * component.
     * 
     * @param componentName
     *            name of the component to query; must be non-null and non-empty
     * @param serviceUnitName
     *            name of the subject service unit; must be non-null and
     *            non-empty
     * @return <code>true</code> if the named service unit is currently
     *         deployed to the named component
     * @throws java.lang.Exception
     */
    public boolean isDeployedServiceUnit(java.lang.String componentName,
            java.lang.String serviceUnitName) throws java.lang.Exception;
	
	/**
	 * TODO shutDown returning xml result instead of SA name
	 * 
     * Shut down the service assembly. This puts the assembly back into the
     * {@link DeploymentServiceMBean#SHUTDOWN} state.
     * 
     * @param serviceAssemblyName
     *            name of the assembly to be shut down; must be non-null and
     *            non-empty
     * @return result / status string giving the results of shutting down each
     *         service unit in the assembly; must be non-null and non-empty
     * @throws java.lang.Exception
     *             if there is no such assembly
     * @throws java.lang.Exception
     *             if the assembly fails to shut down
     */
    public java.lang.String shutDown(java.lang.String serviceAssemblyName)
            throws java.lang.Exception;

	/**
	 * TODO start returning xml result instead of SA name
	 * 
     * Start the service assembly. This puts the assembly into the
     * {@link DeploymentServiceMBean#STARTED} state.
     * 
     * @param serviceAssemblyName
     *            name of the assembly to be started; must be non-null and
     *            non-empty
     * @return result / status string giving the results of starting (and
     *         possibly initializing) each service unit in the assembly; must be
     *         non-null and non-empty
     * @throws java.lang.Exception
     *             if there is no such assembly
     * @throws java.lang.Exception
     *             if the assembly fails to start
     */
    public java.lang.String start(java.lang.String serviceAssemblyName)
            throws java.lang.Exception;
	
	/**
	 * TODO stop returning xml result instead of SA name
	 * 
     * Stop the service assembly. This puts the assembly into the
     * {@link DeploymentServiceMBean#STOPPED} state.
     * 
     * @param serviceAssemblyName
     *            name of the assembly to be stopped; must be non-null and
     *            non-empty
     * @return result / status string giving the results of stopping each
     *         service unit in the assembly; must be non-null and non-empty
     * @throws java.lang.Exception
     *             if there is no such assembly
     * @throws java.lang.Exception
     *             if the assembly fails to stop
     */
    public java.lang.String stop(java.lang.String serviceAssemblyName)
            throws java.lang.Exception;
	
	/**
	 * TODO undeploy returning xml result instead of SA name
	 * 
     * Undeploys the given Service Assembly from the JBI environment.
     * 
     * @param serviceAssemblyName
     *            name of the Service Assembly that is to be undeployed; must be
     *            non-null and non-empty
     * @return Result/Status of the current undeployment; must conform to JBI
     *         management result/status XML schema; must be non-null and
     *         non-empty
     * @throws java.lang.Exception
     *             if compelete undeployment fails
     */
    public java.lang.String undeploy(java.lang.String serviceAssemblyName)
            throws java.lang.Exception;
}
